{% load i18n %}
<div class="well well-small">
    <h3>{% blocktrans %}Image Registry tool:{% endblocktrans %}</h3>
    <br />
    <p>
        {% blocktrans %}Image Registry is used to provide additional information about images for Data Processing.{% endblocktrans %}
    </p>
    <p>
        {% blocktrans %}Specified User Name will be used by Data Processing to apply configs and manage processes on instances.{% endblocktrans %}
    </p>
    <p>
        {% blocktrans %}Tags are used for filtering images suitable for each plugin and each Data Processing version.
        To add required tags, select a plugin and a Data Processing version and click &quot;Add all&quot; button.{% endblocktrans %}
    </p>
    <p>
        {% blocktrans %}You may also add any custom tag.{% endblocktrans %}
    </p>
    <p>
        {% blocktrans %}Unnecessary tags may be removed by clicking a cross near tag's name.{% endblocktrans %}
    </p>
</div>