{% extends "horizon/common/_modal_form.html" %}
{% load i18n horizon humanize %}
{% load url from future %}

{% block form_id %}{% endblock %}
{% block form_action %}{% url 'horizon:project:routers:create' %}?{{ request.GET.urlencode }}{% endblock %}

{% block modal_id %}create_router_modal{% endblock %}
{% block modal-header %}{% trans "Create router" %}{% endblock %}

{% block modal-body %}
  <div class="left">
    <fieldset>
      {% include "horizon/common/_form_fields.html" %}
    </fieldset>
  </div>
{% endblock %}

{% block modal-footer %}
  <input class="btn btn-primary pull-right" type="submit" value="{% trans "Create router" %}" />
  <a href="{% url 'horizon:project:routers:index' %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
