{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Evacuate Host" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Evacuate Host") %}
{% endblock page_header %}

{% block main %}
  {% include 'admin/hypervisors/compute/_evacuate_host.html' %}
{% endblock %}