{% extends "horizon/common/_modal_form.html" %}

{% load url from future %}

{% load i18n %}

{% block form_id %}upload_file{% endblock %}
{% block form_action %}{% url 'horizon:project:data_processing.cluster_templates:upload_file' %}{% endblock %}
{% block form_attrs %}enctype="multipart/form-data"{% endblock %}

{% block modal-header %}{% trans "Upload Template" %}{% endblock %}

{% block modal-body %}
    <div class="left">
        <fieldset>
            {% include "horizon/common/_form_fields.html" %}
        </fieldset>
    </div>
{% endblock %}

{% block modal-footer %}
    <input class="btn btn-primary pull-right" id="upload_file_btn" type="submit" value="{% trans "Upload" %}"/>
    <a href="{% url 'horizon:project:data_processing.cluster_templates:index' %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}