<noscript><h3>{{ step }}</h3></noscript>
<table class="table-fixed">
  <tbody>
    <tr>
      <td class="actions">
          {% include "horizon/common/_form_fields.html" %}
      </td>
      <td class="help_text">
          {{ step.get_help_text }}
      </td>
    </tr>
  </tbody>
</table>
