{% load i18n horizon %}
<div class="well">
<p>
  {% blocktrans %}Add libraries to your job.{% endblocktrans %}
</p>
<p>
  {% blocktrans %}Choose from the list of binaries and click "choose" to add the library to your job.  This can be repeated for additional libraries.{% endblocktrans %}
</p>
</div>