{% load i18n sizeformat parse_date %}
{% load url from future %}

<div class="info row detail">
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ firewall.name|default:_("-") }}</dd>

    <dt>{% trans "Description" %}</dt>
    <dd>{{ firewall.description|default:_("-") }}</dd>

    <dt>{% trans "ID" %}</dt>
    <dd>{{ firewall.id }} </dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ firewall.tenant_id }}</dd>

    <dt>{% trans "Policy ID" %}</dt>
    <dd>
      {% url 'horizon:project:firewalls:policydetails' firewall.firewall_policy_id as policy_url%}
      <a href="{{ policy_url }}">{{ firewall.policy.name|default:firewall.policy.id }}</a>
    </dd>

    <dt>{% trans "Status" %}</dt>
    <dd>{{ firewall.status }}</dd>

    <dt>{% trans "Admin State Up" %}</dt>
    <dd>{{ firewall.admin_state_up|yesno|capfirst }}</dd>
  </dl>
</div>
