{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Manage Security Group Rules" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Manage Security Group Rules: ")|add:security_group.name %}
{% endblock page_header %}

{% block main %}
  {{ table.render }}
{% endblock %}
