{% load i18n sizeformat %}
<h3>{% trans "Job Execution Overview" %}</h3>
<div class="status row detail">
    <dl>
        <dt>{% trans "Status" %}</dt>
        <dd>{{ job_execution.info.status }}</dd>
        <dt>{% trans "Id" %}</dt>
        <dd>{{ job_execution.id }}</dd>
        <dt>{% trans "Job Id" %}</dt>
        <dd>{{ job_execution.job_id }} ({{ object_names.job_name }})</dd>
        <dt>{% trans "Input Id" %}</dt>
        <dd>{{ job_execution.input_id }} ({{ object_names.input_name }})</dd>
        <dt>{% trans "Output Id" %}</dt>
        <dd>{{ job_execution.output_id }} ({{ object_names.output_name }})</dd>
        <dt>{% trans "Cluster Id" %}</dt>
        <dd>{{ job_execution.cluster_id }} ({{ object_names.cluster_name }})</dd>
        <dt>{% trans "Last Updated" %}</dt>
        <dd>{{ job_execution.updated_at }}</dd>
        <dt>{% trans "Started" %}</dt>
        <dd>{{ job_execution.start_time }}</dd>
        <dt>{% trans "Ended" %}</dt>
        <dd>{{ job_execution.end_time }}</dd>
        <dt>{% trans "Return Code" %}</dt>
        <dd>{{ job_execution.return_code }}</dd>
        <dt>{% trans "Oozie Job Id" %}</dt>
        <dd>{{ job_execution.oozie_job_id }}</dd>
        <dt>{% trans "Created" %}</dt>
        <dd>{{ job_execution.created_at }}</dd>
        <dt>{% trans "Tenant Id" %}</dt>
        <dd>{{ job_execution.tenant_id }}</dd>
        <dt>{% trans "Job Configuration" %}</dt>
        <dd>{% for group, vals in job_execution.job_configs.iteritems %}
            <ul><li><span style="font-weight:bold">{{ group }}:</span>
              {%if group == "args" %}
                <ul>{% for val in vals %} <li>{{ val }}</li> {% endfor %}</ul>
              {% else %}
                <ul>{% for key, val in vals.iteritems %} <li>{{ key }} = {{ val }}</li> {% endfor %}</ul>
              {% endif %}
            </li></ul>
            {% endfor %}
        </dd>
    </dl>
</div>
