{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Import Key Pair" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Import Key Pair") %}
{% endblock page_header %}

{% block main %}
  {% include 'project/access_and_security/keypairs/_import.html' %}
{% endblock %}

