{% load i18n sizeformat parse_date %}

<div class="info row detail">
  <hr class="header_rule">
  <dl>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ member.id }}</dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ member.tenant_id }}</dd>

    <dt>{% trans "Pool" %}</dt>
    {% url 'horizon:project:loadbalancers:pooldetails' member.pool_id as pool_url %}
    <dd><a href="{{pool_url}}">{{ member.pool.name_or_id }}</a></dd>

    <dt>{% trans "Address" %}</dt>
    <dd>{{ member.address }}</dd>

    <dt>{% trans "Protocol Port" %}</dt>
    <dd>{{ member.protocol_port }}</dd>

    <dt>{% trans "Weight" %}</dt>
    <dd>{{ member.weight }}</dd>

    <dt>{% trans "Admin State Up" %}</dt>
    <dd>{{ member.admin_state_up|yesno|capfirst }}</dd>

    <dt>{% trans "Status" %}</dt>
    <dd>{{ member.status }}</dd>
  </dl>
</div>
