{% load i18n horizon %}

<p>{% blocktrans %}Choose rule(s) from Available Rules to Selected Rule by push button or drag and drop,
you may change their order by drag and drop as well. {% endblocktrans %}</p>
