{% load i18n sizeformat %}
<h3>{% trans "Data Source Overview" %}</h3>
<div class="status row detail">
    <dl>
        <dt>{% trans "Name" %}</dt>
        <dd>{{ data_source.name }}</dd>
        <dt>{% trans "ID" %}</dt>
        <dd>{{ data_source.id }}</dd>
        <dt>{% trans "Type" %}</dt>
        <dd>{{ data_source.type }}</dd>
        <dt>{% trans "URL" %}</dt>
        <dd>{{ data_source.url }}</dd>
        <dt>{% trans "Tenant id" %}</dt>
        <dd>{{ data_source.tenant_id }}</dd>
        <dt>{% trans "Description" %}</dt>
        <dd>{{ data_source.description|default:_("None") }}</dd>
        <dt>{% trans "Create time" %}</dt>
        <dd>{{ data_source.created_at }}</dd>
    </dl>
</div>
