{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Edit IKE Policy" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Edit IKE Policy") %}
{% endblock page_header %}

{% block main %}
  {% include 'project/vpn/_update_ikepolicy.html' %}
{% endblock %}