{% load i18n %}
<ul>
  <li>{% trans "Object Count: " %}{{ container.container_object_count }}</li>
  <li>{% trans "Size: " %}{{ container.container_bytes_used|filesizeformat }}</li>
  <li>{% trans "Access: " %}
  {% if container.public_url %}
      <a href="{{ container.public_url }}">{% trans "Public" %}</a>
  {% else %}
      {% trans "Private" %}
  {% endif %}
  </li>
</ul>
