{% load i18n sizeformat parse_date %}

<div class="info row detail">
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ ikepolicy.name|default:_("None") }}</dd>

    <dt>{% trans "Description" %}</dt>
    <dd>{{ ikepolicy.description|default:_("None") }}</dd>

    <dt>{% trans "ID" %}</dt>
    <dd>{{ ikepolicy.id }}</dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ ikepolicy.tenant_id }}</dd>

    <dt>{% trans "Authorization algorithm" %}</dt>
    <dd>{{ ikepolicy.auth_algorithm }}</dd>

    <dt>{% trans "Encryption algorithm" %}</dt>
    <dd>{{ ikepolicy.encryption_algorithm }}</dd>

    <dt>{% trans "IKE version" %}</dt>
    <dd>{{ ikepolicy.ike_version }}</dd>

    <dt>{% trans "Lifetime Units" %}</dt>
    <dd>{{ ikepolicy.lifetime.units }}</dd>

    <dt>{% trans "Lifetime Value" %}</dt>
    <dd>{{ ikepolicy.lifetime.value }}</dd>

    <dt>{% trans "Perfect Forward Secrecy" %}</dt>
    <dd>{{ ikepolicy.pfs }}</dd>

    <dt>{% trans "IKE Phase1 negotiation mode" %}</dt>
    <dd>{{ ikepolicy.phase1_negotiation_mode }}</dd>
  </dl>
</div>
