{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans 'Group Management' %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Group Management: ")|add:group.name %}
{% endblock page_header %}

{% block main %}
    {{ group_members_table.render }}
{% endblock %}
