{% load i18n %}
<a href="#" id="flavor_details_{{ id }}" class="link-popover" rel="popover" data-content="
<table class='table table-bordered'>
  <tr><th>{% trans 'VCPUs' %}</th><td>{{ vcpus }}</td></tr>
  <tr><th>{% trans 'RAM' %}</th><td>{{ size_ram }}</td></tr>
  <tr><th>{% trans 'Size' %}</th><td>{{ size_disk }}</td></tr>
</table>
" data-original-title="{% trans "Flavor Details" %}: {{ name }}">{{ name }}</a>
<script type="text/javascript" charset="utf-8">
$(function ()
    { $("#flavor_details_{{ id }}").popover({html:true});
});
</script>
