{% extends 'base.html' %}
{% load i18n %}
{% load url from future %}
{% block title %}{% blocktrans %}Download Key Pair{% endblocktrans %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Download Key Pair") %}
{% endblock page_header %}

{% block main %}
  <div class="modal-header">
    <h3>{% blocktrans %}The key pair &quot;{{ keypair_name }}&quot; should download automatically. If not use the link below.{% endblocktrans %}</h3>
  </div>
  <div class="modal-body clearfix">
    <a href="{% url 'horizon:project:access_and_security:keypairs:generate' keypair_name %}">
      {% blocktrans %}Download key pair &quot;{{ keypair_name}}&quot;{% endblocktrans %}
    </a>
  </div>
  <script type="text/javascript" charset="utf-8">
    $(document).ready(function() {
      document.location = '{% url 'horizon:project:access_and_security:keypairs:generate' keypair_name %}';
    });
  </script>
{% endblock %}
