{% load i18n sizeformat parse_date %}

<div class="info row detail">
  <hr class="header_rule">
  <dl>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ monitor.id }}</dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ monitor.tenant_id }}</dd>

    <dt>{% trans "Type" %}</dt>
    <dd>{{ monitor.type }}</dd>

    <dt>{% trans "Delay" %}</dt>
    <dd>{{ monitor.delay }}</dd>

    <dt>{% trans "Timeout" %}</dt>
    <dd>{{ monitor.timeout }}</dd>

    <dt>{% trans "Max Retries" %}</dt>
    <dd>{{ monitor.max_retries }}</dd>

    {% if monitor.type == 'HTTP' or monitor.type == 'HTTPS' %}
    <dt>{% trans "HTTP Method" %}</dt>
    <dd>{{ monitor.http_method }}</dd>

    <dt>{% trans "URL Path" %}</dt>
    <dd>{{ monitor.url_path }}</dd>

    <dt>{% trans "Expected Codes" %}</dt>
    <dd>{{ monitor.expected_codes }}</dd>
    {%  endif %}

    <dt>{% trans "Admin State Up" %}</dt>
    <dd>{{ monitor.admin_state_up|yesno|capfirst }}</dd>

    <dt>{% trans "Pools" %}</dt>
    {% if monitor.pools %}
    {% for pool in monitor.pools %}
    {% url 'horizon:project:loadbalancers:pooldetails' pool.id as pool_url %}
    <dd><a href="{{pool_url}}">{{ pool.name_or_id }}</a></dd>
    {% endfor %}
    {% else %}
    <dd>{% trans "None" %}</dd>
    {% endif %}
  </dl>
</div>
