<div id="{% block modal_id %}{% endblock %}" class="{% block modal_class %}{% if hide %}modal{% else %}static_page{% endif %}{% endblock %}">
  <div class="{% if hide %}modal-dialog{% endif %}">
    <div class="{% if hide %}modal-content{% endif %}">
      <div class="modal-header">
        {% if hide %}<a href="#" class="close" data-dismiss="modal">&times;</a>{% endif %}
        <h3>{% block modal-header %}{% endblock %}</h3>
      </div>
      <div class="modal-body clearfix">
        {% block modal-body %}{% endblock %}
      </div>
      <div class="modal-footer">{% block modal-footer %}{% endblock %}</div>
    </div>
  </div>
</div>
