{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% load url from future %}

{% block form_id %}update_policy_form{% endblock %}
{% block form_action %}{% url 'horizon:project:firewalls:insertrule' policy_id %}{% endblock %}

{% block modal-header %}{% trans "Insert Rule to Policy" %}{% endblock %}

{% block modal-body %}
<div class="left">
    <fieldset>
    {% include "horizon/common/_form_fields.html" %}
    </fieldset>
</div>
<div class="right">
    <h3>{% trans "Description:" %}</h3>
    <p>{% blocktrans %}Choose the rule you want to insert. Specify
    either the rule you want to insert immediately before, or the rule
    to insert immediately after. If both are specified, the prior
    takes precedence.{% endblocktrans %}</p>
</div>
{% endblock %}

{% block modal-footer %}
  <input class="btn btn-primary pull-right" type="submit" value="{% trans "Save Changes" %}" />
  <a href="{% url 'horizon:project:firewalls:index' %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
