{% extends 'base.html' %}
{% load i18n %}

{% block title %}{% trans "Create Image Custom Property" %}{% endblock %}

{% block page_header %}
  <h2>{% trans "Image" %}: {{image.name}} </h2>
{% endblock page_header %}

{% block main %}
    {% include "admin/images/properties/_create.html" %}
{% endblock %}
