{% extends "horizon/common/_modal.html" %}
{% load i18n %}
{% load url from future %}

{% block modal_id %}extra_specs_modal{% endblock %}
{% block modal-header %}{% trans "Flavor Extra Specs" %}{% endblock %}

{% block modal-body %}
    {{ table.render }}
{% endblock %}

{% block modal-footer %}
  <a href="{% url 'horizon:admin:flavors:index' %}" class="btn btn-default secondary cancel close">{% trans "Close" %}</a>
{% endblock %}

