{% load i18n %}

<noscript><h3>{{ step }}</h3></noscript>
<table class="table-fixed" id="networkListSortContainer">
  <tbody>
    <tr>
      <td class="actions">
        <label id="selected_network_label">{% trans "Selected networks" %}</label>
        <ul id="selected_network" class="networklist">
        </ul>
        <label>{% trans "Available networks" %}</label>
        <ul id="available_network" class="networklist">
        </ul>
      </td>
      <td class="help_text">
          {% include "project/databases/_launch_network_help.html" %}
      </td>
    </tr>
  </tbody>
</table>

<table class="table-fixed" id="networkListIdContainer">
  <tbody>
    <tr>
      <td class="actions">
          <div id="networkListId">
            {% include "horizon/common/_form_fields.html" %}
          </div>
      </td>
      <td class="help_text">
          {{ step.get_help_text }}
      </td>
    </tr>
  </tbody>
</table>


<script>
  if (typeof $ !== 'undefined') {
    horizon.instances.workflow_init($(".workflow"));
  } else {
    addHorizonLoadEvent(function() {
      horizon.instances.workflow_init($(".workflow"));
    });
  }
</script>
