{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Update Network" %}{% endblock %}

{% block page_header %}
    {% include "horizon/common/_page_header.html" with title=_("Update Network Profile") %}
{% endblock page_header %}

{% block main %}
    {% include 'router/nexus1000v/_update_network_profile.html' %}
{% endblock %}
