{% load i18n sizeformat %}

<h3>{% trans "Network Overview" %}</h3>

<div class="info detail">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ network.name|default:_("None") }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ network.id|default:_("None") }}</dd>
    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ network.tenant_id|default:"-" }}</dd>
    <dt>{% trans "Status" %}</dt>
    <dd>{{ network.status|default:_("Unknown") }}</dd>
    <dt>{% trans "Admin State" %}</dt>
    <dd>{{ network.admin_state|default:_("Unknown") }}</dd>
    <dt>{% trans "Shared" %}</dt>
    <dd>{{ network.shared|yesno|capfirst }}</dd>
    <dt>{% trans "External Network" %}</dt>
    <dd>{{ network.router__external|yesno|capfirst }}</dd>
    {% if network.provider__network_type %}
    <dt>{% trans "Provider Network" %}</dt>
    <dd>{% trans "Network Type" %}: {{ network.provider__network_type|default:_("Unknown") }}</dd>
    <dd>{% trans "Physical Network" %}: {{ network.provider__physical_network|default:"-" }}</dd>
    <dd>{% trans "Segmentation ID" %}: {{ network.provider__segmentation_id|default:"-" }}</dd>
    {% endif %}
  </dl>
</div>
