
<script type="text/javascript">
    addHorizonLoadEvent(function () {
        horizon.modals.addModalInitFunction(function (modal) {
          hide_extra_fields();
          function hide_extra_fields() {
            $("[name=job_binary_username]").closest(".form-group").hide();
            $("[name=job_binary_password]").closest(".form-group").hide();
            $("[name=job_binary_file]").closest(".form-group").hide();
            $("[name=job_binary_url]").closest(".form-group").hide();
            $("[name=job_binary_internal]").closest(".form-group").hide();
          }
          $("#id_job_binary_type").change(function() {
            var label = $("#id_job_binary_url_label");
            var bin_file = $("[name=job_binary_file]").closest(".form-group");
            var bin_choice = $("[name=job_binary_internal]").closest(".form-group");
            var bin_url = $("[name=job_binary_url]").closest(".form-group");
            var username = $("[name=job_binary_username]").closest(".form-group");
            var password = $("[name=job_binary_password]").closest(".form-group");
            var script_name = $("[name=job_binary_script_name]").closest(".form-group");
            var script_text = $("[name=job_binary_script]").closest(".form-group");
            switch($(this).val()) {
              case "internal-db":
                label.html("internal-db://");
                username.hide();
                password.hide();
                bin_file.show();
                bin_choice.show();
                bin_url.hide();
                script_name.hide();
                script_text.hide();
                break;
              case "swift":
                username.show();
                password.show();
                bin_file.hide();
                bin_choice.hide();
                bin_url.show();
                label.html("swift://");
                script_name.hide();
                script_text.hide();
                break;
            }
          });
          $("#id_job_binary_type").change();

          $("[name=job_binary_internal]").change(function() {
            var bin_file = $("[name=job_binary_file]").closest(".form-group");
            var bin_choice = $("[name=job_binary_internal]").closest(".form-group");
            var bin_url = $("[name=job_binary_url]").closest(".form-group");
            var script_name = $("[name=job_binary_script_name]").closest(".form-group");
            var script_text = $("[name=job_binary_script]").closest(".form-group");
            switch($(this).val()) {
              case "%%%NEWSCRIPT%%%":
                bin_file.hide();
                script_name.show();
                script_text.show();
                break;
              case "%%%UPLOADFILE%%%":
                bin_file.show();
                script_name.hide();
                script_text.hide();
                break;
              default:
                bin_file.hide();
                script_name.hide();
                script_text.hide();
                break;
            }
          });
          $("#job_binary_internal").change();
       });
    });
</script>
