{% extends "horizon/client_side/template.html" %}
{% load horizon %}

{% block id %}balloon_port{% endblock %}

{% block template %}
{% jstemplate %}
<table class="detailInfoTable">
  <caption>[[interfaces_label]]</caption>
  <tbody>
    [[#port]]
    <tr>
      <th>
        <span title="[[id]]">
          <a href="[[url]]">[[id]]</a>
        </span>
      </th>
      <td>[[ip_address]]</td>
      <td>[[device_owner]]</td>
      <td>
        <span class="[[port_status_class]]">[[port_status]]</span>
      </td>
      <td class="delete">
        [[#is_interface]]
        <button class="delete-port btn btn-danger btn-xs" data-router-id="[[router_id]]"  data-port-id="[[id]]">[[interface_label]]</button>
        [[/is_interface]]
      </td>
    </tr>
    [[/port]]
  </tbody>
</table>
{% endjstemplate %}
{% endblock %}
