{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% load url from future %}

{% block form_id %}copy_object_form{% endblock %}
{% block form_action %}{% url 'horizon:project:containers:object_copy' container_name object_name %}{% endblock %}

{% block modal-header %}{% trans "Copy Object" %}: {{ object_name }}{% endblock %}

{% block modal-body %}
<div class="left">
    <fieldset>
    {% include "horizon/common/_form_fields.html" %}
    </fieldset>
</div>
<div class="right">
    <h3>{% trans "Description" %}:</h3>
    <p>{% trans "Make a new copy of an existing object to store in this or another container. You may additionally specify the path within the selected container where the new copy should be stored." %}</p>
</div>
{% endblock %}

{% block modal-footer %}
    <input class="btn btn-primary pull-right" type="submit" value="{% trans "Copy Object" %}" />
    <a href="{% url 'horizon:project:containers:index' container_name|add:'/' %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
