{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Usage Overview" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Overview") %}
{% endblock page_header %}

{% block main %}
  {% if monitoring %}
    <div id="monitoring">
      <h3>{% trans "Monitoring" %}: </h3>
      <ul id="external_links">
        {% for link in monitoring %}
          <li><a target="_blank" href="{{ link.1 }}">{{ link.0 }}</a></li>
        {% endfor %}
      </ul>
    </div>
  {% endif %}

  {% if simple_tenant_usage_enabled %}
    {% include "horizon/common/_usage_summary.html" %}
    {{ table.render }}
  {% endif %}
{% endblock %}
