{% load i18n %}
<p>{% blocktrans %}Automatic: Entire disk is single partition and automatically resizes.{% endblocktrans %}</p>
<p>{% blocktrans %}Manual: Faster build times but requires manual partitioning.{% endblocktrans %}</p>
