{% load i18n sizeformat parse_date %}
{% load url from future %}

<div class="info row detail">
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ policyrule.name|default:_("-") }}</dd>

    <dt>{% trans "Description" %}</dt>
    <dd>{{ policyrule.description|default:_("-") }}</dd>

    <dt>{% trans "ID" %}</dt>
    <dd>{{ policyrule.id }}</dd>

    <dt>{% trans "Project ID" %}</dt>
    <dd>{{ policyrule.tenant_id }}</dd>
	
	<dt>{% trans "Enabled" %}</dt>
    <dd>{{ policyrule.enabled|default:_("-") }}</dd>
    
    <dt>{% trans "Policy Actions" %}</dt>
    <dd>
      {% for action in actions%}
	  {{action.name}}
      {% url 'horizon:project:contracts:policyactiondetails' action as action_url %}
      {{ action.position }} : <a href="{{ action_url }}">{{ action.id }}</a><br>
      {% endfor %}
    </dd>
    <dt>{% trans "Policy Classifier" %}</dt>
    <dd>
      {% for classifier in classifiers %}
      {{classifier.name}}:
      {% url 'horizon:project:contracts:policyclassifierdetails' classifier.id as classifier_url %}
      <a href="{{ classifier_url }}">{{ classifier.id }}</a><br>
      {% trans "-" %}
      {% endfor %}
    </dd>
  </dl>
</div>
