{% load i18n sizeformat parse_date %}
{% load url from future %}

<h3>{% trans "Volume Overview" %}</h3>

<div class="info row detail">
  <h4>{% trans "Info" %}</h4>
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ volume.name }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ volume.id }}</dd>
    {% if volume.description %}
    <dt>{% trans "Description" %}</dt>
    <dd>{{ volume.description }}</dd>
    {% endif %}
    <dt>{% trans "Status" %}</dt>
    <dd>{{ volume.status|capfirst }}</dd>
  </dl>
</div>

<div class="specs row detail">
  <h4>{% trans "Specs" %}</h4>
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Size" %}</dt>
    <dd>{{ volume.size }} {% trans "GB" %}</dd>
    <dt>{% trans "Created" %}</dt>
    <dd>{{ volume.created_at|parse_date }}</dd>
  </dl>
</div>

<div class="status row detail">
  <h4>{% trans "Attachments" %}</h4>
  <hr class="header_rule">
  <dl>
  {% for attachment in volume.attachments %}
      <dt>{% trans "Attached To" %}</dt>
      <dd>
        {% url 'horizon:project:instances:detail' attachment.server_id as instance_url%}
        <a href="{{ instance_url }}">{{ attachment.instance.name }}</a>
        <span> {% trans "on" %} {{ attachment.device }}</span>
      </dd>
  {% empty %}
      <dt>{% trans "Attached To" %}</dt>
      <dd><em>{% trans "Not attached" %}</em></dd>
  {% endfor %}
  </dl>
</div>

{% if volume.volume_image_metadata %}
<div class="status row detail">
  <h4>{% trans "Volume Source" %}</h4>
  <hr class="header_rule">
  <dl>
      <dt>{% trans "Image" %}</dt>
      <dd>
        {% url 'horizon:project:images:images:detail' volume.volume_image_metadata.image_id as image_url%}
        <a href="{{ image_url }}">{{ volume.volume_image_metadata.image_name }}</a>
      </dd>
  </dl>
</div>
{% endif %}

<div class="status row detail">
  <h4>{% trans "Metadata" %}</h4>
  <hr class="header_rule">
  <dl>
  {% for key, value in volume.metadata.items %}
    <dt>{{ key }}</dt>
    <dd>{{ value }}</dd>
  {% endfor %}
  </dl>
</div>
