{% extends 'base.html' %}
{% load i18n %}
{% load url from future %}
{% block title %}{% trans 'Usage Report' %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Daily Usage Report Per Project")%}
{% endblock page_header %}

{% block main %}
    <div class="table_actions">
      <a href="{{ csv_url }}"><button class="btn btn-default btn-sm" ><span class="glyphicon glyphicon-download"></span>{% trans 'Download CSV Summary' %}</button></a>
      <a href="{% url 'horizon:admin:metering:index' %}"><button class="btn btn-default btn-sm">{% trans 'Back' %}</button></a>
    </div>
{% for table in tables %}
    {{ table.render }}
{% endfor %}
{% if not tables %}
    <p>
    {% trans 'No data available' %}
    </p>
{% endif %}
{% endblock %}

