#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests import helpers


class TestDashboardHelp(helpers.TestCase):

    def test_dashboard_help_redirection(self):
        """Verifies Help link redirects to the right URL."""

        self.home_pg.go_to_help_page()
        for handle in self.driver.window_handles:
            self.driver.switch_to_window(handle)

        self.assertEqual(self.conf.dashboard.help_url,
                         self.driver.current_url,
                         "help link did not redirect to the right URL")

        self.driver.close()
        for handle in self.driver.window_handles:
            self.driver.switch_to_window(handle)
