{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Network Detail"%}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Network Detail: ")|add:network.name %}
{% endblock page_header %}

{% block main %}
  {% include "project/networks/_detail_overview.html" %}
  <hr>
  <div id="subnets">
    {{ subnets_table.render }}
  </div>
  <div id="ports">
    {{ ports_table.render }}
  </div>
  {% if dhcp_agent_support %}
    <div id="agents">
      {{ agents_table.render }}
    </div>
  {% endif %}
{% endblock %}
