{% load i18n sizeformat parse_date %}
{% load url from future %}

<h3>{% trans "Volume Snapshot Overview" %}</h3>

<div class="info row detail">
  <h4>{% trans "Info" %}</h4>
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Name" %}</dt>
    <dd>{{ snapshot.name }}</dd>
    <dt>{% trans "ID" %}</dt>
    <dd>{{ snapshot.id }}</dd>
    {% if snapshot.description %}
    <dt>{% trans "Description" %}</dt>
    <dd>{{ snapshot.description }}</dd>
    {% endif %}
    <dt>{% trans "Status" %}</dt>
    <dd>{{ snapshot.status|capfirst }}</dd>
    <dt>{% trans "Volume" %}</dt>
    <dd>
      <a href="{% url 'horizon:project:volumes:volumes:detail' snapshot.volume_id %}">
        {% if volume.name %}
          {{ volume.name }}
        {% else %}
          {{ snapshot.volume_id }}
        {% endif %}
      </a>
    </dd>
  </dl>
</div>

<div class="specs row detail">
  <h4>{% trans "Specs" %}</h4>
  <hr class="header_rule">
  <dl>
    <dt>{% trans "Size" %}</dt>
    <dd>{{ snapshot.size }} {% trans "GB" %}</dd>
    <dt>{% trans "Created" %}</dt>
    <dd>{{ snapshot.created_at|parse_date }}</dd>
  </dl>
</div>
