{% extends "base.html" %}
{% load i18n %}
{% block title %}{% trans "Resize Database Volume" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Resize Database Volume") %}
{% endblock %}

{% block main %}
  {% include "project/databases/_resize_volume.html" %}
{% endblock %}