<img src="{{ stack_image }}" width="35px" height="35px" />
<div id="stack_info">
    <h3>{{ stack.stack_name }}</h3>
    <p class="error">{{ stack.stack_status_desc }}</p>
</div>
<div class="clear"></div>
{% if stack.stack_status == 'CREATE_FAILED' %}
    <p class="error">{{ stack.stack_status_reason }}</p>
{% else %}
    <p>{{ stack.stack_status_desc }}</p>
{% endif %}
{% for output in stack.outputs %}
    {% if output.output_key == 'WebsiteURL' %}
    <a href="{{ output.output_value }}">{{ output.description }}</a>
    {% endif %}
{% endfor %}