{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Data Processing" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Job Executions") %}
{% endblock page_header %}

{% block main %}

<div class="job_executions">
    {{ job_executions_table.render }}
</div>

<script type="text/javascript">
    addHorizonLoadEvent(function () {

        horizon.modals.addModalInitFunction(function (modal) {
            if ($(modal).find(".nav-tabs").find("li").size() == 1) {
                // hide tab bar for plugin/version modal wizard
               $('div#modal_wrapper ul.nav-tabs').hide();
            }

            $(".hidden_nodegroups_field").val("");
            $(".hidden_configure_field").val("");

            lower_limit = 0;
            $(".count-field").change();

            if ($(modal).find(".hidden_create_field").length > 0) {
                var form = $(".hidden_create_field").closest("form");
                var successful = false;
                form.submit(function (e) {
                  var newElement = $("<a id='hiddenbutton' class='btn btn-small ajax-modal' style='display:none'/>");
                  var oldHref = $("#job_executions__action_delete")[0].href;
                  var plugin = $("#id_plugin_name option:selected").val();
                  var version = $("#id_" + plugin + "_version option:selected").val();
                  var job_id = $("#id_job_id").val();
                  var job_execution_id = $("#id_job_execution_id").val();
                  form.find(".close").click();
                  newElement.attr("href", "launch-job-new-cluster?" +
                    "plugin_name=" + encodeURIComponent(plugin) +
                    "&hadoop_version=" + encodeURIComponent(version) +
                    "&job_id=" + encodeURIComponent(job_id) +
                    "&job_execution_id=" + encodeURIComponent(job_execution_id));
                  newElement.appendTo("#main_content");
                  $("#hiddenbutton").click();
                  return false;
                });
                $(".plugin_version_choice").closest(".form-group").hide();
            }

            //display version for selected plugin
            $(document).on('change', '.plugin_name_choice', switch_versions);
            function switch_versions() {
                $(".plugin_version_choice").closest(".form-group").hide();
                var plugin = $(this);
                $("." + plugin.val() + "_version_choice").closest(".form-group").show();
            }
            $(".plugin_name_choice").change();
        });

    });
</script>

{% endblock %}
