{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Data Processing" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Data Processing - Clusters") %}
{% endblock page_header %}

{% block main %}

<div class="cluster_templates">
    {{ clusters_table.render }}
</div>

<script type="text/javascript">
    addHorizonLoadEvent(function () {

        horizon.modals.addModalInitFunction(function (modal) {
            if ($(modal).find(".nav-tabs").find("li").size() == 1) {
                // hide tab bar for plugin/version modal wizard
               $('div#modal_wrapper ul.nav-tabs').hide();
            }

            $(".hidden_nodegroups_field").val("");
            $(".hidden_configure_field").val("");

            lower_limit = 0;
            $(".count-field").change();

            if ($(modal).find(".hidden_create_field").length > 0) {
                var form = $(".hidden_create_field").closest("form");
                var successful = false;
                form.submit(function (e) {
                        var oldHref = $(".configure-cluster-btn")[0].href;
                        var plugin = $("#id_plugin_name option:selected").val();
                        var version = $("#id_" + plugin + "_version option:selected").val();
                        form.find(".close").click();
                        $(".configure-cluster-btn")[0].href = oldHref +
                                "?plugin_name=" + encodeURIComponent(plugin) +
                                "&hadoop_version=" + encodeURIComponent(version);
                        $(".configure-cluster-btn").click();
                        $(".configure-cluster-btn")[0].href = oldHref;
                        return false;
                });
                $(".plugin_version_choice").closest(".form-group").hide();
            }

            //display version for selected plugin
            $(document).on('change', '.plugin_name_choice', switch_versions);
            function switch_versions() {
                $(".plugin_version_choice").closest(".form-group").hide();
                var plugin = $(this);
                $("." + plugin.val() + "_version_choice").closest(".form-group").show();
            }
            $(".plugin_name_choice").change();
        });

    });
</script>

{% endblock %}
