{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Update Role" %}{% endblock %}

{% block page_header %}
  {# to make searchable false, just remove it from the include statement #}
  {% include "horizon/common/_page_header.html" with title=_("Update Role") %}
{% endblock page_header %}

{% block main %}
    {% include 'identity/roles/_update.html' %}
{% endblock %}
