{% load branding i18n %}
{% load url from future %}
{% load context_selection %}

<h1 class="brand"><a href="{% site_branding_link %}">{% site_branding %}</a></h1>

<div class="context-box">
  <div class="dropdown context-selection">
    <button class="btn btn-default btn-sm dropdown-toggle" data-toggle="dropdown">
        {% show_overview %}
    </button>
    <div class="dropdown-menu">
      <div class="context-lists">
        {% show_domain_list %}
        {% show_project_list %}
        {% show_region_list %}
      </div>
      <div class="footer disabled">
        <span class="glyphicon glyphicon-cog"></span>
        {% trans "Filter selections" %}
      </div>
    </div><!-- end of dropdown-menu -->
  </div><!-- end of context-selection -->
</div><!-- end of context-box -->

<div id="user_info" class="pull-right">
  <div id="profile_editor_switcher" class="dropdown switcher_bar" tabindex='1'>
    <a class="dropdown-toggle" data-toggle="dropdown" href="#profile_editor_switcher">
      <div>{{ request.user.username }}</div>
    </a>
    <ul id="editor_list" class="dropdown-menu">
      <li class='divider'></li>
      <li><a href="{% url 'horizon:settings:user:index' %}">{% trans "Settings" %}</a></li>
      {% if HORIZON_CONFIG.help_url %}
        <li><a href="{{ HORIZON_CONFIG.help_url }}" target="_new">{% trans "Help" %}</a></li>
      {% endif %}
    </ul>
  </div>

  <a href="{% url 'logout' %}">{% trans "Sign Out" %}</a>
  {% include "horizon/common/_region_selector.html" %}
</div>

