{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Image Custom Properties" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Image Custom Properties: ")|add:image.name|default:_("Image Custom Properties:") %}
{% endblock page_header %}

{% block main %}
  {{ table.render }}
{% endblock %}
