{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% load url from future %}

{% block form_id %}{% endblock %}
{% block form_action %}{% url 'horizon:project:volumes:volumes:retype' volume.id %}{% endblock %}

{% block modal_id %}retype_volume_modal{% endblock %}
{% block modal-header %}{% trans "Change Volume Type" %}{% endblock %}

{% block modal-body %}
  <div class="left">
    <fieldset>
    {% include "horizon/common/_form_fields.html" %}
    </fieldset>
  </div>
  <div class="right">
    <h3>{% trans "Description:" %}</h3>
    <p>{% blocktrans %}
        From here you can change the volume type of a volume after its creation.
        This is equivalent to the <tt>cinder retype</tt> command.
       {% endblocktrans %}
    </p>
    <p>{% blocktrans %}
        The "Volume Type" selected must be different from the current volume type.
       {% endblocktrans %}
    </p>
    <p>{% blocktrans %}
        The "Migration Policy" is only used if the volume retype cannot be
        completed. If the "Migration Policy" is "On Demand", the back end will
        perform volume migration. Note that migration may take a significant
        amount of time to complete, in some cases hours.
       {% endblocktrans %}
    </p>
  </div>
{% endblock %}

{% block modal-footer %}
  <input class="btn btn-primary pull-right" type="submit" value="{% trans "Change Volume Type" %}" />
  <a href="{% url 'horizon:project:volumes:index' %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
