{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% load url from future %}

{% block form_id %}{% endblock %}
{% block form_action %}{% url 'horizon:project:volumes:volumes:create_snapshot' volume_id %}{% endblock %}

{% block modal_id %}create_volume_snapshot_modal{% endblock %}
{% block modal-header %}{% trans "Create Volume Snapshot" %}{% endblock %}

{% block modal-body %}
  <div class="left">
    <fieldset>
    {% include "horizon/common/_form_fields.html" %}
    </fieldset>
  </div>
  <div class="right quota-dynamic">
    {% include "project/volumes/volumes/_snapshot_limits.html" with usages=usages snapshot_quota=True %}
  </div>
{% endblock %}

{% block modal-footer %}
  {% if attached %}
    <input class="btn btn-primary btn-warning pull-right" type="submit" value="{% trans "Create Volume Snapshot (Force)" %}" />
  {% else %}
    <input class="btn btn-primary pull-right" type="submit" value="{% trans "Create Volume Snapshot" %}" />
  {% endif %}
  <a href="{% url 'horizon:project:volumes:index' %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
