<div id="{% block modal_id %}{% endblock %}" class="{% block modal_class %}{% if hide %}modal{% else %}static_page{% endif %}{% endblock %}">
  <div class="{% if hide %}modal-dialog{% endif %}">
    <div class="{% if hide %}modal-content{% endif %}">
      <div class="modal-header">
        {% if hide %}<a href="#" class="close" data-dismiss="modal">&times;</a>{% endif %}
        <h3>{% block modal-header %}{% endblock %}</h3>
      </div>
      {% if table %}
      <div class="modal-body">
      {{ table.render }}
      </div>
      <hr />
      {% endif %}
      <form id="{% block form_id %}{% endblock %}"
            ng-controller="{% block ng_controller %}DummyCtrl{% endblock %}"
            name="{% block form_name %}{% endblock %}"
            autocomplete="{% block autocomplete %}{% if form.no_autocomplete %}off{% endif %}{% endblock %}"
            class="{% block form_class %}{% endblock %}"
            action="{% block form_action %}{% endblock %}"
            method="{% block form-method %}POST{% endblock %}"
            {% if add_to_field %}data-add-to-field="{{ add_to_field }}"{% endif %} {% block form_attrs %}{% endblock %}>{% csrf_token %}
        <div class="modal-body clearfix">
        {% comment %}
        These fake fields are required to prevent Chrome v34+ from autofilling form.
        {% endcomment %}
        {% if form.no_autocomplete %}
        <input type="text" name="fake_email" value="" style="display: none" />
        <input type="password" name="fake_password" value="" style="display: none" />
        {% endif %}
        {% block modal-body %}
            <fieldset>
            {% include "horizon/common/_form_fields.html" %}
            </fieldset>
        {% endblock %}
        </div>
        <div class="modal-footer">{% block modal-footer %}{% endblock %}</div>
      </form>
    </div>
  </div>
</div>
{% block modal-js %}
{% endblock %}
