
.context-selection {
  display: inline-block;

  .dropdown-toggle {
    border: none;
    background: transparent;
    font-size: $font-size-base;
    color: $gray;

    .context-overview {
      padding: 0 0.25em;
    }
  }

  .dropdown-menu {
    padding: 0;
  }

  .dropdown-header {
    font-size: $font-size-small
  }

  .context-lists {
    display: table;
    padding: 0.5em 1em;
  }

  .footer {
    width: 100%;
    margin-top: 1em;
    padding: 0.4em 1em;
    background: whitesmoke;
    border-top: 1px solid $gray-light;
    text-align: right;
  }

  .disabled {
    cursor: not-allowed;
    color: $gray-light;
  }

  a {
    cursor: pointer;
    text-decoration: none;
    white-space: nowrap;
  }

  em {
    font-size: $font-size-large;
    font-weight: bold;
    vertical-align: middle;
  }

  ul {
    display: table-cell;
    padding-left: 1em;

    li {
      padding-left: 0;
      font-size: $font-size-base;
      text-overflow: ellipsis;
      overflow: hidden;
      max-width: 160px;
    }

    li .glyphicon-ok {
      margin-left: 0.2em;
    }
  }

  ul:not(:last-child) {
    border-right: 1px solid $gray-light;
  }
}
