{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Job Details" %}{% endblock %}

{% block page_header %}
    {% include "horizon/common/_page_header.html" with title=_("Job Details") %}
{% endblock page_header %}

{% block main %}
    <div class="row">
        <div class="col-sm-12">
        {{ tab_group.render }}
        </div>
    </div>
{% endblock %}