{% extends "horizon/common/_modal.html" %}
{% load i18n %}
{% load url from future %}

{% block modal-header %}{% trans "Container Details" %}{% endblock %}

{% block modal-body %}
<div class="info row detail clearfix">
  <dl>
    <dt>{% trans "Container Name" %}</dt>
    <dd>{{ container.name }}</dd>
    <dt>{% trans "Container Access" %}</dt>
    {% if container.public_url %}
    <dd>{% trans "Public" %}</dd>
    <dt>{% trans "Public URL" %}</dt>
    <dd>{{ container.public_url }}</dd>
    {% else %}
    <dd>{% trans "Private" %}</dd>
    {% endif %}
    <dt>{% trans "Object Count" %}</dt>
    <dd>{{ container.container_object_count }}</dd>
    <dt>{% trans "Size" %}</dt>
    <dd>{{ container.container_bytes_used|filesizeformat }}</dd>
  </dl>
</div>
{% endblock %}

{% block modal-footer %}
  <a href="{% url 'horizon:project:containers:index' %}" class="btn btn-default secondary cancel close">{% trans "Close" %}</a>
{% endblock %}
