{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}
{% load url from future %}

{% block form_id %}update_networkprofile_form{% endblock %}
{% block form_action %}{% url 'horizon:router:nexus1000v:update_network_profile profile_id' %}{% endblock %}

{% block modal-header %}{% trans "Edit Network Profile" %}{% endblock %}

{% block modal-body %}
    <div class="left">
        <fieldset>
            {% include "horizon/common/_form_fields.html" %}
        </fieldset>
    </div>
    <div class="right">
        <h3>{% trans "Description:" %}</h3>
        <p>{% trans "You may update the editable properties of your network profile here." %}</p>
    </div>
{% endblock %}

{% block modal-footer %}
    <input class="btn btn-primary pull-right" type="submit" value="{% trans "Save Changes" %}" />
    <a href="{% url horizon:router:nexus1000v:index %}" class="btn btn-default secondary cancel close">{% trans "Cancel" %}</a>
{% endblock %}
