{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Hypervisor Servers" %}{% endblock %}

{% block page_header %}
{% include "horizon/common/_page_header.html" with title=_("Hypervisor Servers") %}
{% endblock page_header %}

{% block main %}
{{ table.render }}
{% endblock %}
