{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Host Aggregates" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Host Aggregates") %}
{% endblock page_header %}

{% block main %}
  <div id="host-aggregates">
      {{ host_aggregates_table.render }}
  </div>

  <div id="availability-zones">
      {{ availability_zones_table.render }}
  </div>
{% endblock %}
